﻿using System.Collections.Generic;
using System.Net.Sockets;

namespace SocketLBR
{
    public class BufferManager
    {
        int numBytes; //the total number of bytes controlled by the buffer pool
        byte[] buffer; //the underlying byte array maintained by the Buffer Manager

        Stack<int> freeIndexPool;
        int currentIndex;
        int bufferSize;

        public BufferManager(int totalBytes, int bufferSize)
        {
            this.numBytes = totalBytes;
            this.currentIndex = 0;
            this.bufferSize = bufferSize;
            freeIndexPool = new Stack<int>();
        }

        //Allocates buffer space used by the buffer pool
        public void InitBuffer()
        {
            buffer = new byte[bufferSize];
        }

        public bool SetBuffer(SocketAsyncEventArgs args)
        {
            if (freeIndexPool.Count > 0)
                args.SetBuffer(buffer, freeIndexPool.Pop(), bufferSize);
            else
            {
                if (numBytes - bufferSize < currentIndex)
                    return false;
                args.SetBuffer(buffer, currentIndex, bufferSize);
                currentIndex += bufferSize;
            }
            return true;
        }

        public void FreeBuffer(SocketAsyncEventArgs args)
        {
            freeIndexPool.Push(args.Offset);
            args.SetBuffer(null, 0, 0);
        }
    }
}
