﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net.Sockets;

namespace SocketListenerServer
{
    delegate void ProcessData(SocketAsyncEventArgs args);

    internal sealed class Token : IDisposable
    {
        private Socket connection;


        private StringBuilder sb;


        private Int32 currentIndex;


        /// <summary>
        /// Class constructor.
        /// </summary>
        /// <param name="connection">Socket to accept incoming data.</param>
        /// <param name="bufferSize">Buffer size for accepted data.</param>
        internal Token(Socket connection, Int32 bufferSize)
        {
            this.connection = connection;
            this.sb = new StringBuilder(bufferSize);
        }


        /// <summary>
        /// Accept socket.
        /// </summary>
        internal Socket Connection
        {
            get { return this.connection; }
        }


        //处理缓冲数据
        internal void ProcessUserRegisterData()
        {
            sb.Length = 0;
            this.currentIndex = 0;
        }

        /// <summary>
        /// Process data received from the client.
        /// </summary>
        /// <param name="args">SocketAsyncEventArgs used in the operation.</param>
        internal void ProcessData(SocketAsyncEventArgs args)
        {
            // Get the message received from the client.
            String received = this.sb.ToString();

            int handle = (int)this.Connection.Handle;
            //int handle=args.
            //TODO Use message received to perform a specific operation.
            Console.WriteLine("Received From Handle[{2}] Message: {0} . The server has read {1} bytes.", received, received.Length, handle.ToString());

            //Byte[] sendBuffer = Encoding.UTF8.GetBytes("Return " + received);
            Byte[] sendBuffer = Encoding.UTF8.GetBytes(received);
            args.SetBuffer(sendBuffer, args.Offset, sendBuffer.Length);


            // Clear StringBuffer, so it can receive more data from a keep-alive connection client.
            sb.Length = 0;
            this.currentIndex = 0;
        }


        /// <summary>
        /// Set data received from the client.
        /// </summary>
        /// <param name="args">SocketAsyncEventArgs used in the operation.</param>
        internal void SetData(SocketAsyncEventArgs args)
        {
            Int32 count = args.BytesTransferred;


            if ((this.currentIndex + count) > this.sb.Capacity)
            {
                //int intSplitCount = sb.Capacity - currentIndex < 0 ? 0 : sb.Capacity - currentIndex;
                //sb.Append(Encoding.UTF8.GetString(args.Buffer, args.Offset, intSplitCount));
                //this.currentIndex = this.sb.Capacity;
                return;
                //throw new ArgumentOutOfRangeException("count",
                //    String.Format(CultureInfo.CurrentCulture, "Adding {0} bytes on buffer which has {1} bytes, the listener buffer will overflow.", count, this.currentIndex));
            }
            else
            {
                sb.Append(Encoding.UTF8.GetString(args.Buffer, args.Offset, count));
                this.currentIndex += count;
            }
        }


        #region IDisposable Members


        /// <summary>
        /// Release instance.
        /// </summary>
        public void Dispose()
        {
            try
            {
                this.connection.Shutdown(SocketShutdown.Send);
            }
            catch (Exception)
            {
                // Throw if client has closed, so it is not necessary to catch.
            }
            finally
            {
                this.connection.Close();
            }
        }


        #endregion


    }
}
