﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net.Sockets;

namespace SocketServer
{
    public sealed class BufferManager
    {
        private byte[] buffer;  //基础Byte数组

        private int bufferSize; //Byte数组大小

        private int currentIndex; //Byte数组当前索引

        private Stack<int> freeIndexPool; //缓冲区池

        private int numBytes; //池中单个缓冲区字节总数

        /// <summary>
        /// 初始化缓冲区
        /// </summary>
        /// <param name="totalBytes"><缓冲池中字节总数/param>
        /// <param name="bufferSize">缓冲池中单个区的大小</param>
        public BufferManager(int totalBytes, int bufferSize)
        {
            this.numBytes = totalBytes;
            this.currentIndex = 0;
            this.bufferSize = bufferSize;
            this.freeIndexPool = new Stack<int>();
        }

        /// <summary>
        /// 从缓冲池中移除SocketAsyncEventArgs对象所占用的缓冲区,这将释放缓冲区到缓冲池中
        /// </summary>
        /// <param name="args">要删除缓冲区的SocketAsyncEventArgs对象</param>
        public void FreeBuffer(SocketAsyncEventArgs args)
        {
            this.freeIndexPool.Push(args.Offset);
            args.SetBuffer(null, 0, 0);
        }

        /// <summary>
        /// 分配缓冲池所使用的缓冲区空间
        /// </summary>
        public void InitBuffer()
        {
            this.buffer = new byte[this.numBytes]; //为每隔SocketAsyncEventArgs对象创建巨大的缓冲区
        }

        /// <summary>
        /// 将SocketAsyncEventArgs对象的数据放入缓冲区
        /// </summary>
        /// <param name="args">需要分配缓冲区的SocketAsyncEventArgs对象</param>
        /// <returns>如果设置成功,为True; 反之,为False</returns>
        public bool SetBuffer(SocketAsyncEventArgs args)
        {
            if (this.freeIndexPool.Count > 0)
                args.SetBuffer(this.buffer, this.freeIndexPool.Pop(), this.bufferSize);
            else
            {
                if ((this.numBytes - this.bufferSize) < this.currentIndex)
                    return false;

                args.SetBuffer(this.buffer, this.currentIndex, this.bufferSize);
                //string received = Encoding.ASCII.GetString(this.buffer, 0, this.bufferSize);
                this.currentIndex += this.bufferSize;
            }
            return true;
        }
    }
}
