﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net.Sockets;
using System.Net;
using System.Threading;
using System.Windows.Forms;
using System.Text.RegularExpressions;

namespace SocketServer
{
    public class Server
    {
        Socket serverSocket;
        SocketAsyncEventArgs socketAsyncEventArgs;
        SocketAsyncEventArgsPool readWritePool;
        HandleMessage handleMessage;
        BufferManager buffeManager;

        const int PrefixSize = 11;

        public void Init(int port,int connections,int receiveBufferSize)
        {
            buffeManager = new BufferManager(receiveBufferSize * connections * 2, receiveBufferSize);

            buffeManager.InitBuffer();

            readWritePool = new SocketAsyncEventArgsPool(connections);

            SocketAsyncEventArgs socketAsyncEventArgsPooling;
            for (int i = 0; i < connections; i++)
            {
                socketAsyncEventArgsPooling = new SocketAsyncEventArgs();
                socketAsyncEventArgsPooling.Completed += readEventArgsIO_Completed;

                buffeManager.SetBuffer(socketAsyncEventArgsPooling);
                readWritePool.Push(socketAsyncEventArgsPooling);
            }

            handleMessage = new HandleMessage();

            IPAddress[] addressList = Dns.GetHostEntry(Environment.MachineName).AddressList;
            IPEndPoint localEndPoint = new IPEndPoint(addressList[addressList.Length - 1], port);

            this.serverSocket = new Socket(localEndPoint.AddressFamily, SocketType.Stream, ProtocolType.Tcp);

            //区分IP地址{IPv4和IPv6}
            if (localEndPoint.AddressFamily == AddressFamily.InterNetworkV6)
            {
                this.serverSocket.SetSocketOption(SocketOptionLevel.IPv6, (SocketOptionName)27, false);
                this.serverSocket.Bind(new IPEndPoint(IPAddress.IPv6Any, localEndPoint.Port));
            }
            else
            {
                //侦听器绑定端口
                this.serverSocket.Bind(localEndPoint);
            }

            this.serverSocket.Listen(100);

            StartAccept(null);
        }

        private void StartAccept(SocketAsyncEventArgs acceptSocketAsyncEventArgs)
        {
            if (acceptSocketAsyncEventArgs == null)
            {
                acceptSocketAsyncEventArgs = new SocketAsyncEventArgs();
                acceptSocketAsyncEventArgs.Completed += socketAsyncEventArgs_Completed;
            }
            else
            {
                acceptSocketAsyncEventArgs.AcceptSocket = null;
            }

            Boolean willRaiseEvent = this.serverSocket.AcceptAsync(acceptSocketAsyncEventArgs);
            if (!willRaiseEvent)
            {
                this.ProcessAccept(acceptSocketAsyncEventArgs);
            }
        }

        private void socketAsyncEventArgs_Completed(object sender, SocketAsyncEventArgs e)
        {
            ProcessAccept(e);
        }

        private void readEventArgsIO_Completed(object sender, SocketAsyncEventArgs e)
        {
            switch (e.LastOperation)
            {
                case SocketAsyncOperation.Receive:
                    this.ProcessReceive(e);
                    break;
                case SocketAsyncOperation.Send:
                    //this.ProcessSend(e);
                    break;
                default:
                    throw new ArgumentException("The last operation completed on the socket was not a receive or send");
            }
        }

        //这里主要是讲连接的客户端分发给SocketAsyncEventArgs对象,进行处理
        private void ProcessAccept(SocketAsyncEventArgs e)
        {
            if (e.SocketError != SocketError.Success)
            {
                this.StartAccept(e);
                e.AcceptSocket.Close();
                readWritePool.Push(e);
            }

            SocketAsyncEventArgs readEventArgs = this.readWritePool.Pop();
            //SocketAsyncEventArgs readEventArgs = new SocketAsyncEventArgs();
            readEventArgs.UserToken = e.AcceptSocket;
            readEventArgs.AcceptSocket = e.AcceptSocket;

            Console.WriteLine("---------------------------------------------------");
            Console.WriteLine("Client Connected {0}",e.AcceptSocket.RemoteEndPoint);

            //这个地方要注意,一定得设置
            //如果不设置,将会导致出现"对象未设置到引用的实例"错误
            //如果设置的值过小,将会导致缓冲区内数据的不完整
            //正确的值的设置方法是: 连接数*接收的对象的长度*2
            //readEventArgs.SetBuffer(new byte[100000], 0, 100000);
            //readEventArgs.Completed += readEventArgsIO_Completed;

          

           // Thread.Sleep(1);
            //监听客户端,是否有接收数据传来
            Boolean willRaiseEvent = e.AcceptSocket.ReceiveAsync(readEventArgs);

            if (!willRaiseEvent)
            {
                this.ProcessReceive(readEventArgs);
            }

            //继续侦听
            this.StartAccept(e);
        }

        private void ProcessReceive(SocketAsyncEventArgs e)
        {
            #region
            ////if (e.BytesTransferred > 0)
            ////{
            //    if (e.SocketError == SocketError.Success)
            //    {
            //        StringBuilder sbStr = new StringBuilder(1024);
            //        int bytesTransferred = e.BytesTransferred;

            //        Console.WriteLine("receiving data, {0} bytes", bytesTransferred);

            //        //e.AcceptSocket = e.UserToken as Socket;
            //        e.UserToken = e.AcceptSocket;

            //        string received = Encoding.ASCII.GetString(e.Buffer, e.Offset, bytesTransferred);

            //        string pattern = @"(?<=^\[totallength=)(\d+)(?=\])";

            //        int totalLength = 0;

            //        if (Regex.IsMatch(received, pattern))
            //        {
            //            Match match = Regex.Match(received, pattern);
            //            Console.WriteLine("The total length is:{0} bytes", match.Value);
            //            totalLength = Int32.Parse(match.Value);
            //        }

            //        int tmpLength = totalLength.ToString().Length + 14;
            //        int receivedLength = received.Length;
            //        if (receivedLength - tmpLength == totalLength)
            //        {
            //            //接收完毕

            //            sbStr.Append(received.Substring(tmpLength));
            //            Console.WriteLine("Received Complete:{0}", sbStr.ToString());
            //            string[] msgArray = handleMessage.GetActualString(sbStr.ToString());
            //            foreach (var msg in msgArray)
            //            Console.WriteLine("After Split:{0}", msg);
            //            //Array.Clear(e.Buffer, e.Offset, e.BytesTransferred);


            //        }
            //        if (receivedLength - tmpLength < totalLength)
            //        {
            //            //未接收完全,则继续接收
            //            if (!e.AcceptSocket.ReceiveAsync(e))
            //                ProcessReceive(e);
            //        }
            //        if (receivedLength - tmpLength > totalLength)
            //        {
            //            //接收过多,说明接收完毕,但是下一个块又追加了过来,我们只处理完整的部分


            //        }



            //        //if (e.AcceptSocket.Available == 0)
            //        //{

            //        //}
            //        //else if (!e.AcceptSocket.ReceiveAsync(e))
            //        //{
            //        //   //Array.Clear(e.Buffer, e.Offset, e.BytesTransferred);
            //        //   ProcessReceive(e);
            //        //}

            //        // Finished, then push back
            //        //CloseClient(e);
            //    }
            //    else
            //    {
            //        Console.WriteLine("User disconnected.");
            //        e.AcceptSocket = null;
            //        buffeManager.FreeBuffer(e);
            //        readWritePool.Push(e);
            //        if (readWritePool.Count == 1)
            //        {
            //            StartAccept(null);
            //        }
            //    }
            ////}
#endregion

            if (e.BytesTransferred > 0)
            {
                if (e.SocketError == SocketError.Success)
                {
                    Socket socket = (Socket)e.UserToken;
                    if (socket.Available == 0)
                    {
                        string str = Encoding.ASCII.GetString(e.Buffer,e.Offset,e.BytesTransferred);

                        Console.WriteLine(str);
                        
                    }
                    else if (!socket.ReceiveAsync(e))
                    {
                        ProcessReceive(e);
                    }
                }
            }
        }

        private void CloseClient(SocketAsyncEventArgs e)
        {
            e.AcceptSocket.Shutdown(SocketShutdown.Receive);
            e.AcceptSocket.Close();
            readWritePool.Push(e);
        }

        private void ProcessSend(SocketAsyncEventArgs e)
        {

        }
    }
}

/**
 * The FLOW:
 *  StartAccept -->  socketAsyncEventArgs_Completed　-> ProcessAccept
 * 
 * **/
